/* blocking-driver1.c
 * 
 * I/O műveletek blokkolása 1. példa.
 * 
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <linux/slab.h>
#include <linux/device.h>

#define DEVICE_NAME "hello"
static int major_num=120;
#define CLASS_NAME "helloclass"
static struct class* hello_class;

wait_queue_head_t wait_queue;

static ssize_t hello_write(struct file *pfile, const char *buf, size_t count, loff_t *ppos)
{
  printk("Wake..\n");
  wake_up_interruptible(&wait_queue);
  return count;
}

static ssize_t hello_read(struct file *pfile, char *buf, size_t count, loff_t *ppos)
{
  if(pfile->f_flags & O_NONBLOCK)
  {
    return -EAGAIN;
  }

  printk("Sleep...\n");
  interruptible_sleep_on(&wait_queue);
  printk("End of sleep\n");
  return 0;
}

static int hello_open(struct inode *inode, struct file *pfile)
{
  try_module_get(THIS_MODULE);
  printk(DEVICE_NAME " open.\n");
  return 0;
}

static int hello_close(struct inode *inode, struct file *pfile)
{
  printk(DEVICE_NAME " close.\n");
  module_put(THIS_MODULE);
  return 0;
}

static struct file_operations hello_fops =
{
  owner:	THIS_MODULE,
  read:	hello_read,
  write:	hello_write,
  open:	hello_open,
  release:	hello_close
};

static int __init hello_init(void)
{
  int res;
  struct device* err;

  init_waitqueue_head(&wait_queue);
  
  res = register_chrdev(major_num, DEVICE_NAME, &hello_fops);
  if(res<0)
  {
    printk(DEVICE_NAME " a major szam: %d nem elerheto\n",major_num);
    return res;
  }
  if(major_num==0) major_num=res;

  hello_class = class_create(THIS_MODULE, CLASS_NAME);
  err = device_create(hello_class, NULL, MKDEV(major_num, 0), NULL, DEVICE_NAME);
  
  printk(DEVICE_NAME " betoltve. Major szam: %d\n",major_num);
  return 0;
}

static void __exit hello_exit(void)
{
  device_destroy(hello_class, MKDEV(major_num, 0));
  class_unregister(hello_class);
  class_destroy(hello_class);
  
  unregister_chrdev(major_num, DEVICE_NAME);
  printk(DEVICE_NAME " felszabaditva.\n");	
}

module_init(hello_init);
module_exit(hello_exit);

MODULE_DESCRIPTION("Hello driver");
MODULE_LICENSE("GPL");
