/* devattr.c
 *
 * Egyszerű eszköz sysfs attribútumokkal.
 *  
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/platform_device.h>

#define DRV_NAME "devattr"

static int i = 0;

/* Az attribútum lekérdező függvénye */
static ssize_t show_num(struct device *dev, struct device_attribute *attr, char *buf)
{
  return sprintf(buf, "%d\n", i);
}

/* Az attribútum beállító függvénye */
static ssize_t store_num(struct device *dev, struct device_attribute *attr, const char *buf, size_t count)
{
  sscanf(buf, "%d", &i);
  return count;
}

/* Az attribútum összeállítása */
DEVICE_ATTR(num, S_IRUGO | S_IWUSR, show_num, store_num);

static int __devinit devattr_probe(struct platform_device *pdev)
{
  dev_info(&pdev->dev, "probe");
  device_create_file(&pdev->dev, &dev_attr_num);
  
  return 0;
}

static int __devexit devattr_remove(struct platform_device *pdev)
{
  dev_info(&pdev->dev, "remove");
  device_remove_file(&pdev->dev, &dev_attr_num);

  return 0;
}

static struct platform_driver my_driver = {
  .probe = devattr_probe,
  .remove = __devexit_p(devattr_remove),
  .driver = {
    .name = DRV_NAME,
    .owner = THIS_MODULE,
  }
};

static struct platform_device* my_dev;

static int __init my_init(void)
{
  int err;

  err = platform_driver_register(&my_driver);
  if (err)
    return err;
  
  my_dev = platform_device_register_simple(DRV_NAME, -1, NULL, 0);
  if(IS_ERR(my_dev))
  {
    platform_driver_unregister(&my_driver);
    return PTR_ERR(my_dev);
  }
  
  return 0;
}

static void __exit my_exit(void)
{  
  platform_device_unregister(my_dev);
  platform_driver_unregister(&my_driver);
}

module_init(my_init);
module_exit(my_exit);

MODULE_DESCRIPTION("Device attr");
MODULE_LICENSE("GPL");
