/* helloirq1.c
 * 
 * Egyszerű megszakításkezelő példa tasklet használatával.
 * 
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/interrupt.h>
#include <asm/irq.h>

unsigned long counter1 = 0;
unsigned long counter2 = 0;
unsigned int cirq = 1;
static unsigned int devid = 33;

static DEFINE_SPINLOCK(hello_lock);

void hello_tasklet_proc(unsigned long);
DECLARE_TASKLET(hello_tasklet, hello_tasklet_proc, 0);

irqreturn_t counter_irq(int irq, void *dev_id)
{
  counter1++;
  tasklet_schedule(&hello_tasklet);
  return IRQ_NONE;
}

void hello_tasklet_proc(unsigned long nothing)
{
  unsigned long flags;
  spin_lock_irqsave(&hello_lock, flags);
  counter2=counter1;
  spin_unlock_irqrestore(&hello_lock, flags);
}

int procfile_read(char* buffer, char** buffer_location, off_t offset, int
buffer_length, int *eof, void *procdata)
{
  //unsigned long flags;
  int len;

  /*spin_lock_irqsave(&hello_lock, flags);
  len=sprintf(buffer, "Szamlalo: %lu\n", counter1);
  spin_unlock_irqrestore(&hello_lock, flags);*/

  spin_lock(&hello_lock);
  len=sprintf(buffer, "Szamlalo: %lu\n", counter2);
  spin_unlock(&hello_lock);
  return len;     
}

static int __init hello_init(void)
{
  struct proc_dir_entry *hello_proc_ent;

  if(request_irq(cirq, counter_irq, IRQF_SHARED, "Hello megszakitaskezelo", &devid))
  {
    printk(KERN_WARNING "A %d megszakitas foglalt\n", cirq);
    cirq = -1;
  }
    
  hello_proc_ent = create_proc_entry("hello", S_IFREG | S_IRUGO, 0);

  if(hello_proc_ent)
  {
    hello_proc_ent->nlink = 1;
    hello_proc_ent->read_proc = procfile_read;
  }
  return 0;
}

static void __exit hello_exit(void)
{
  if(cirq >= 0) free_irq(cirq, &devid);
  
  remove_proc_entry("hello",0);
}

module_init(hello_init);
module_exit(hello_exit);

MODULE_DESCRIPTION("Hello megszakitaskezelo");
MODULE_LICENSE("GPL");
