/* helloproc.c
 * 
 * Egyszerű kernel modul amely a /proc állományrendszerben létrehoz egy
 * olvasható állományt.
 * 
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>

/* Az állomány olvasását kezelő függvény. */
int procfile_read(char* buffer, char** buffer_location, off_t offset, int
buffer_length, int *eof, void *procdata)
{
  int len;
  
  /* Az egyszerűség kedvéért nem engedünk részleges olvasást. */        
  if(offset)
  {
    *eof = 1;
    return 0;
  }
  
  /* Feltöltjük a buffert és visszatérünk az adat hosszával. */
  len=sprintf(buffer, "Hello proc vilag!\n");
  return len;     
}

static int __init hello_init(void)
{
  struct proc_dir_entry *hello_proc_ent;
  
  /* Dinamikusan hozzuk létre a proc állomány bejegyzést. */
  hello_proc_ent = create_proc_entry("hello", S_IFREG | S_IRUGO, 0);
  
  /* Beállítjuk a link számlálót és a lekezelő függvényt. */
  if(!hello_proc_ent)
  {
    remove_proc_entry("hello", 0);
    printk("Error: A /proc/hello nem letrehozhato.\n");
    return -ENOMEM;
  }
  
  hello_proc_ent->nlink = 1;
  hello_proc_ent->read_proc = procfile_read;
  printk("A /proc/hello letrehozva.\n");
  
  return 0;
}

static void __exit hello_exit(void)
{
  /* Megsemmisítjük a proc állomány bejegyzést. */
  remove_proc_entry("hello",0);
  printk("A /proc/hello eltavolitva.\n");
}

module_init(hello_init);
module_exit(hello_exit);

MODULE_DESCRIPTION("Hello proc");
MODULE_LICENSE("GPL");
