/* ioctl-driver.c
 *
 * Egyszerű eszközmeghajtó ioctl kezeléssel.
 *  
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/ioctl.h>
#include <linux/slab.h>
#include <linux/device.h>
#include <linux/mutex.h>

#include "ioctl-driver.h"

#define DEVICE_NAME "hello"
static int major_num=120;
#define CLASS_NAME "helloclass"
static struct class* hello_class;

int counter=0;
static DEFINE_MUTEX(lock);

static ssize_t hello_write(struct file *pfile, const char *buf, size_t count, loff_t *ppos)
{
  printk(DEVICE_NAME " write.\n");
  return 0;
}

static ssize_t hello_read(struct file *pfile, char *buf, size_t count, loff_t *ppos)
{
  printk(DEVICE_NAME " read.\n");
  return 0;
}

static int hello_open(struct inode *inode, struct file *pfile)
{
  try_module_get(THIS_MODULE);
  printk(DEVICE_NAME " open.\n");
  return 0;
}

static int hello_close(struct inode *inode, struct file *pfile)
{
  printk(DEVICE_NAME " close.\n");
  module_put(THIS_MODULE);
  return 0;
}

/* Az ioctl parancsot kezelő függvény. */
long hello_ioctl(struct file *pfile, unsigned int cmd, unsigned long arg)
{
  int res=0;
  
  /* A parancs paraméterezésének ellenőrzése. */
  if(_IOC_TYPE(cmd) != HELLO_IOC_MAGIC) return -ENOTTY;
  if(_IOC_DIR(cmd) & _IOC_READ)
  {
    if(!access_ok(VERIFY_READ, (void *)arg, _IOC_SIZE(cmd))) return -EFAULT;
  }
  if(_IOC_DIR(cmd) & _IOC_WRITE)
  {
    if(!access_ok(VERIFY_WRITE, (void *)arg, _IOC_SIZE(cmd)))  return -EFAULT;
  }
 
  /* A parancs értelmezése és végrehajtása. */
  switch(cmd)
  {
    case HELLO_IOC_INCREASE:
      if(mutex_lock_interruptible(&lock)) return -ERESTARTSYS;
      counter++;
      mutex_unlock(&lock);
      break;
    case HELLO_IOC_DECREASE:
      if(mutex_lock_interruptible(&lock)) return -ERESTARTSYS;
      counter--;
      mutex_unlock(&lock);
      break;
    case HELLO_IOC_GET:
      if(mutex_lock_interruptible(&lock)) return -ERESTARTSYS;
      res=put_user(counter, (int*)arg);
      mutex_unlock(&lock);
      break;
    case HELLO_IOC_SET:
      if(mutex_lock_interruptible(&lock)) return -ERESTARTSYS;
      res=get_user(counter, (int*)arg);
      mutex_unlock(&lock);
      break;
    default:
      return -ENOIOCTLCMD;
  }
  return res;
}

static struct file_operations hello_fops =
{
  owner:	THIS_MODULE,
  read:	hello_read,
  write:	hello_write,
  open:	hello_open,
  release:	hello_close,
  unlocked_ioctl:	hello_ioctl
};

static int __init hello_init(void)
{
  int res;
  struct device* err;

  res = register_chrdev(major_num, DEVICE_NAME, &hello_fops);
  if(res<0)
  {
    printk(DEVICE_NAME " a major szam: %d nem elerheto\n",major_num);
    return res;
  }
  if(major_num==0) major_num=res;

  hello_class = class_create(THIS_MODULE, CLASS_NAME);
  err = device_create(hello_class, NULL, MKDEV(major_num, 0), NULL, DEVICE_NAME);
  
  printk(DEVICE_NAME " betoltve. Major szam: %d\n",major_num);
  return 0;
}

static void __exit hello_exit(void)
{
  device_destroy(hello_class, MKDEV(major_num, 0));
  class_unregister(hello_class);
  class_destroy(hello_class);
  
  unregister_chrdev(major_num, DEVICE_NAME);
  printk(DEVICE_NAME " felszabaditva.\n");	
}

module_init(hello_init);
module_exit(hello_exit);

MODULE_DESCRIPTION("Hello driver");
MODULE_LICENSE("GPL");
