/* multi-dev2.c 
 *
 * Karakteres eszközvezérlő minor szám kezeléssel.
 * Az eltérő minor számok esetén eltérő szöveget ad vissza.
 * 
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <linux/device.h>

#define DEVICE_NAME "hello"
static int major_num=120;
#define CLASS_NAME "helloclass"
static struct class* hello_class;

#define MSG_LEN 7
static char* msgs[]={ "Hello!", "Howdy!" };

static ssize_t hello_write(struct file *pfile, const char *buf, size_t count, loff_t *ppos);
static ssize_t hello_read1(struct file *pfile, char *buf, size_t count, loff_t *ppos);
static ssize_t hello_read2(struct file *pfile, char *buf, size_t count, loff_t *ppos);
static int hello_open(struct inode *inode, struct file *pfile);
static int hello_close(struct inode *inode, struct file *pfile);

static struct file_operations hello_fops =
{
  owner:	THIS_MODULE,
  read:	hello_read1,
  write:	hello_write,
  open:	hello_open,
  release:	hello_close
};

static struct file_operations hello1_fops =
{
  owner:	THIS_MODULE,
  read:	hello_read1,
  write:	hello_write,
  open:	hello_open,
  release:	hello_close
};

static struct file_operations hello2_fops =
{
  owner:	THIS_MODULE,
  read:	hello_read2,
  write:	hello_write,
  open:	hello_open,
  release:	hello_close
};

struct file_operations* hello_fops_array[] =
{
  &hello1_fops,
  &hello2_fops,
};

static ssize_t hello_write(struct file *pfile, const char *buf, size_t count, loff_t *ppos)
{
  char* str=kmalloc(count+1,GFP_KERNEL);
  if(!copy_from_user(str,buf,count))
  {
    str[count]=0;
    printk(DEVICE_NAME " write: %s\n",str);
  }
  kfree(str);
  return count;
}

static ssize_t hello_read1(struct file *pfile, char *buf, size_t count, loff_t *ppos)
{
  int len;
  len=count;
  if(count>MSG_LEN) len=MSG_LEN;
  if(copy_to_user(buf, msgs[0], len)) return -EFAULT;
  return len;
}

static ssize_t hello_read2(struct file *pfile, char *buf, size_t count, loff_t *ppos)
{
  int len;
  len=count;
  if(count>MSG_LEN) len=MSG_LEN;
  if(copy_to_user(buf, msgs[1], len)) return -EFAULT;
  return len;
}

static int hello_open(struct inode *inode, struct file *pfile)
{
  int num;

  num = MINOR(inode->i_rdev);
  if(num >= 2) return -ENODEV;

  pfile->f_op = hello_fops_array[num];

  try_module_get(THIS_MODULE);
  printk(DEVICE_NAME " open.\n");
  return 0;
}

static int hello_close(struct inode *inode, struct file *pfile)
{
  printk(DEVICE_NAME " close.\n");
  module_put(THIS_MODULE);
  return 0;
}

static int __init hello_init(void)
{
  int res;
  struct device* err;

  res = register_chrdev(major_num, DEVICE_NAME, &hello_fops);
  if(res < 0)
  {
    printk(DEVICE_NAME " a major szam: %d nem elerheto\n", major_num);
    return res;
  }
  if(major_num == 0) major_num = res;

  hello_class = class_create(THIS_MODULE, CLASS_NAME);
  err = device_create(hello_class, NULL, MKDEV(major_num, 0), NULL, DEVICE_NAME"0");
  err = device_create(hello_class, NULL, MKDEV(major_num, 1), NULL, DEVICE_NAME"1");
  
  printk(DEVICE_NAME " betoltve. Major szam: %d\n", major_num);
  return 0;
}

static void __exit hello_exit(void)
{
  device_destroy(hello_class, MKDEV(major_num, 0));
  device_destroy(hello_class, MKDEV(major_num, 1));
  class_unregister(hello_class);
  class_destroy(hello_class);

  unregister_chrdev(major_num, DEVICE_NAME);
  printk(DEVICE_NAME " felszabaditva.\n");	
}

module_init(hello_init);
module_exit(hello_exit);

MODULE_DESCRIPTION("Hello driver");
MODULE_LICENSE("GPL");
